@@ -228,10 +228,15 @@ def _calculate_feature_stats(feature_list, prepared, serialization_file):  # pyl
     # Calculate, min, max and mean vector for each feature with
     # normalization
     start = 0
-    with open(serialization_file, 'w', newline='') as csvfile:
+    mode = 'w'
+    arguments = {'newline': ''}
+    if sys.version_info.major < 3:
+        mode += 'b'
+        arguments = {}
+    with open(serialization_file, mode, **arguments) as csvfile:
         spamwriter = csv.writer(csvfile,
-                                delimiter=';',
-                                quotechar='"',
+                                delimiter=str(';'),
+                                quotechar=str('"'),
                                 quoting=csv.QUOTE_MINIMAL)
         for feature in feature_list:
             end = start + feature.get_dimension()
